#! /usr/bin/env python
# -*- coding: UTF-8 -*-

"""
arp requet에 대한 arp 응답을 직접적으로 전송하는 module
"""

from scapy.all import *


def arp_monitor_callback(pkt):
	"""
	arp 패킷 유형 분석
	tp0 08:00:27:f4:9c:09 20.0.0.1
	tp1 08:00:27:0d:56:93 20.0.0.1
	###[ ARP ]###
	     hwtype    = 0x1	#1 for ethernet
	     ptype     = 0x800	#the same of Ethernet header field carrying IP datagram!
	     hwlen     = 6		#length in bytes of hardware addresses (6 bytes for ethernet(mac))
	     plen      = 4		#length in bytes of logical addresses (4 bytes for IP)
	     op        = is-at	#1=request; 2=reply; 3/4=RARP req/reply
	     hwsrc     = 08:00:27:38:62:de
	     psrc      = 30.0.0.1
	     hwdst     = 08:00:27:0d:56:93
	     pdst      = 20.0.0.1
	"""

    server_iface = 'tp1'
    server_mac = '08:00:27:38:62:de'
    server_ip = '1.1.1.3'

    dut_mac = '08:00:27:0d:56:93'

    if ((ARP in pkt) and (pkt[ARP].op == (1)) and
        (pkt[ARP].pdst == server_ip) and (pkt[Ether].src == dut_mac)):
        print 'got the packet'

        #make ether info
        eth = Ether(dst=pkt[Ether].src, src=server_mac)

        #make arp info (use swap, mac info)
        arp = ARP(hwdst=pkt[ARP].hwsrc,
                  pdst=pkt[ARP].psrc,
                  hwsrc=server_mac,
                  psrc=server_ip,
                  op=2)

        #make packet
        arp_res = eth / arp

        #send arp response
        sendp(arp_res, iface=server_iface)

        #for debug
        pkt.show()
        arp_res.show()

        exit(0)


sniff(prn=arp_monitor_callback, filter="arp", store=0)
