#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
import os
import optparse
import time
import ctypes
from nps_xml import Xml2TcConverter
from nps_common import TcListInfo
from nps_process import nps_process_main
from nps_file_log import *
from multiprocessing import Process, Manager, Lock


def main():
    parser = optparse.OptionParser('usage%prog -f <TC file name>')
    parser.add_option('-f', dest='tcFileName', type='string', help='TC file name')

    (options, args) = parser.parse_args()

    if len(sys.argv) <= 1:
        parser.print_help()
        exit(0)

    #xml -> tc 변환하는 클래스 가져오기.
    tc_converter = Xml2TcConverter()

    if options.tcFileName == None:
        print 'tc File is not exist'
        exit(0)

    client_tc_list_info = TcListInfo("client")
    server_tc_list_info = TcListInfo("server")

    #xml 파일을 읽어서 클래스 인스턴스로 채워넣음.
    tc_converter.analyze_xml_tc_file(options.tcFileName, client_tc_list_info, server_tc_list_info)

    #시작 지점 logging
    server_iface = server_tc_list_info.get_interface_name()
    client_iface = client_tc_list_info.get_interface_name()
    server_start_time = write_report_init(server_iface, options.tcFileName)
    client_start_time = write_report_init(client_iface, options.tcFileName)

    client_process = Process(target=nps_process_main, args=(client_tc_list_info, ))
    server_process = Process(target=nps_process_main, args=(server_tc_list_info, ))

    client_process.start()
    server_process.start()

    #프로세스가 처음 동작하는 시간과 recv스레드의 대기를 위한 시간 지연
    time.sleep(2)

    #프로세스 종료 대기
    server_process.join()
    client_process.join()

    #종료지점 logging
    write_report_close(server_iface, server_start_time)
    write_report_close(client_iface, client_start_time)


if __name__ == '__main__':
    main()
